/* Sun Cluster Data Services Builder template version 1.0 */
/*
 * cups_probe.c - Probe for cups
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <rgm/libdsdev.h>
#include <scha_err.h>

#define SVC_CONNECT_TIMEOUT_PCT         80

int
main(int argc, char *argv[])
{
scds_handle_t	scds_handle;
scha_err_t 	err_code;
int		probe_timeout, connect_timeout, disconnect_timeout;
int		port, ip, probe_result;
char		*wrtBuff, *rdBuff;
hrtime_t	ht1, ht2;
unsigned long	dt;
scds_netaddr_list_t *netaddr;
char *hostname;
int sock_tcp;

/* Initialize handle. */
if (scds_initialize(&scds_handle, argc, argv) != SCHA_ERR_NOERR){
        scds_syslog(LOG_ERR, "Data Service handle failed to initialize!");
	return (1);
}

/* Get the ip addresses available for this resource */
if (scds_get_netaddr_list(scds_handle, &netaddr)) {
	scds_syslog(LOG_ERR,"No network address resource in resource group.");
	scds_close(&scds_handle);
	return (1);
}

/* Return an error if there are no network resources */
if (netaddr == NULL || netaddr->num_netaddrs == 0) {
	scds_syslog(LOG_ERR,"No network address resource in resource group.");
	scds_close(&scds_handle);
	return (1);
}

/*
 * Get the timeout from the extension props. This means that
 * each probe iteration will get a full timeout on each network
 * resource without chopping up the timeout between all of the
 * network resources configured for this resource.
 */

/* Set timeouts */
probe_timeout = scds_get_ext_probe_timeout(scds_handle);
connect_timeout = (probe_timeout * SVC_CONNECT_TIMEOUT_PCT)/100;
disconnect_timeout = ((100 - SVC_CONNECT_TIMEOUT_PCT) * probe_timeout)/100;

for (;;) {
	/*
	 * sleep for a duration of thorough_probe_interval between
	 *  successive probes.
	 */
	scds_fm_sleep(scds_handle, scds_get_rs_thorough_probe_interval(scds_handle));


	/*
	 * Now probe all netaddress we use.
	 * For each of the netaddress that is probed,
	 * compute the failure history.
	 */
	probe_result = 0;
	/*
	 * Iterate through all the netaddrs.
	 */
	for (ip = 0; ip < netaddr->num_netaddrs; ip++) {
		/*
		 * Grab the hostname and port on which the
		 * health has to be monitored.
		 */
		hostname = netaddr->netaddrs[ip].hostname;
		port = netaddr->netaddrs[ip].port_proto.port;
		ht1 = gethrtime(); /* Latch probe start time */

		/* Connect to the cups server. */

               	err_code = scds_fm_tcp_connect(scds_handle, &sock_tcp, hostname, port, connect_timeout);
               	if( err_code != SCHA_ERR_NOERR ){
                	scds_syslog(LOG_ERR,"Host connection Failed! Error: %d -  %s",err_code, scha_strerror(err_code));
			/* since probe is unable to connect to the print server, clients cannot
                         * print which is considered a complete failure.
                         */
			probe_result = SCDS_PROBE_COMPLETE_FAILURE;
                	ht2 = gethrtime();

                	/* Convert to milliseconds */
                	dt = (ht2 - ht1) / 1e6;

			/* Since there was a complete failure, need to take some action. */
			(void) scds_fm_action(scds_handle, probe_result, dt);
               	}

		/* Once connected, should do something to ensure that the cups server is
		   responsive. Maybe able to connect, but the cups server may not be able
                   to do anything.
		*/

               	err_code = scds_fm_tcp_disconnect(scds_handle, sock_tcp, disconnect_timeout);
               	if( err_code != SCHA_ERR_NOERR ){
               		scds_syslog(LOG_ERR,"Host Disconnection Failed! Error: %d - %s",err_code, scha_strerror(err_code));
			probe_result = SCDS_PROBE_COMPLETE_FAILURE/2;
               	}

		ht2 = gethrtime();

		/* Convert to milliseconds */
		dt = (ht2 - ht1) / 1e6;


		/*
		 * Compute failure history and take action if needed
		 */
		(void) scds_fm_action(scds_handle, probe_result, dt);
	}	/* Each netaddr */

    } 	/* Keep probing forever */
}
