/* Sun Cluster Data Services Builder template version 1.0 */
/*
* cups_svc_start.c - Start method for cups
*
* The start method for cups.
*/


#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <strings.h>
#include <syslog.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <scha.h>
#include <rgm/libdsdev.h>
#include <errno.h>

/*
 * The initial timeout allowed  for the cups dataservice to
 * be fully up and running. We will wait for for 3 % (SVC_WAIT_PCT)
 * of the start_timeout time before probing the service.
 */
#define SVC_WAIT_PCT            3

/*
 * We need to use 80% of probe_timeout to connect to the port and the
 * remaining time is used to disconnect from port in the svc_probe function.
 */
#define SVC_CONNECT_TIMEOUT_PCT         80

/*
 * This value will be used as disconnect timeout, if there is no
 * time left from the probe_timeout.
 */

#define SVC_DISCONNECT_TIMEOUT_SECONDS          2


int main(int argc, char *argv[])
{

char start_cli[256];
char *cups_cmd="/sbin/cupsd";
char *cups_args="";
char *cups_homedir;
int rc=0;
int err, log_facility;
int sock_tcp;
int start_timeout, probe_timeout, wait_time;
struct stat stat_buf;
scds_handle_t	scds_handle;
scds_netaddr_list_t *netaddr_list;
scha_err_t err_code;
scha_extprop_value_t *cups_ext_prop;

/* Process all the arguments that have been passed to us from RGM */

if (scds_initialize(&scds_handle, argc, argv) != SCHA_ERR_NOERR) {
	scds_syslog(LOG_ERR, "Data Service handle failed to initialize!");
	return (1);
}


start_timeout = scds_get_rs_start_timeout(scds_handle);

err_code = scds_get_ext_property(scds_handle, "Probe_timeout", SCHA_PTYPE_INT, &cups_ext_prop);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Failed to get Config directory! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}
probe_timeout = cups_ext_prop->val.val_int;

err_code = scds_get_ext_property(scds_handle, "CUPS_dir", SCHA_PTYPE_STRINGARRAY, &cups_ext_prop);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Failed to get Config directory! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}

cups_homedir=cups_ext_prop->val.val_strarray->str_array[0];
strcpy(start_cli,cups_homedir);
strcat(start_cli,cups_cmd);

/* Get config file if specified. */
err_code = scds_get_ext_property(scds_handle, "Conf_file", SCHA_PTYPE_STRINGARRAY, &cups_ext_prop);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Failed to get Configuration File! Error: %s",scha_strerror(err_code));
	scds_close(&scds_handle);
	return(err_code);
}
if(strlen(cups_ext_prop->val.val_strarray->str_array[0]) > 0){
	(void) strcpy(cups_args,cups_ext_prop->val.val_strarray->str_array[0]);
}

scds_free_ext_property(cups_ext_prop);

/* Check the startup command for this method. */
if((stat(start_cli, &stat_buf) !=0)){
	scds_syslog(LOG_ERR, "Cupsd command is not available.");
        scds_close(&scds_handle);
        return(1);
}
if(!(stat_buf.st_mode & S_IXUSR)){
        scds_syslog(LOG_ERR, "Cupsd command is not executeable.");
        scds_close(&scds_handle);
        return(1);
}
/* Add configuration file arg to start cmd. */
if(strlen(cups_args) > 0){
        strcat(start_cli, " -c ");
        strcat(start_cli, cups_homedir);
        strcat(start_cli, cups_args);
        printf("Start cmd w args %s\n",start_cli);
}

/* Start the data service, if it fails return with an error */
err_code = scds_pmf_start(scds_handle, SCDS_PMF_TYPE_SVC, 0, start_cli,-1); 
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR," Cupsd Failed to start! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}

/* Wait SVC_WAIT_PCT (percentage of) START_TIMEOUT, for application
to startup. */

err_code = scds_svc_wait(scds_handle, (scds_get_rs_start_timeout(scds_handle) * SVC_WAIT_PCT)/100);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"scds_svc_wait: Failed to start! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}

/* Quick probe of port to ensure application is running. */
err_code = scds_get_netaddr_list(scds_handle, &netaddr_list);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"No network address! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}


err_code = scds_fm_tcp_connect(scds_handle, &sock_tcp, netaddr_list->netaddrs[0].hostname, netaddr_list->netaddrs[0].port_proto.port, (probe_timeout * SVC_CONNECT_TIMEOUT_PCT)/100);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Host connection Failed! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}

err_code = scds_fm_tcp_disconnect(scds_handle, sock_tcp, SVC_DISCONNECT_TIMEOUT_SECONDS);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Host Disconnection Failed! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}

/* Finished Probe free netaddr_list. */
(void) scds_free_netaddr_list(netaddr_list);

/* Made it to here, send notice to syslog. */
scds_syslog(LOG_NOTICE, "CUPS has started");

/* Free up the Environment resources that were allocated */
(void) scds_close(&scds_handle);
/* Return successful. */
return (0);
}
