/* Sun Cluster Data Services Builder template version 1.0 */
/*
 * cups_validate.c - validate method for cups
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <strings.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <errno.h>
#include <dlfcn.h>
#include <scha.h>
#include <rgm/libdsdev.h>

/*
 * Check to make sure that the properties have been set properly.
 */

#define CMD_SIZE 256

int
main(int argc, char *argv[])
{
scds_handle_t   scds_handle;
scha_err_t	err_code;
scha_err_t	hasp_check_p;
scha_extprop_value_t *cups_ext_prop;
scds_hasp_status_t hasp_status;
scds_net_resource_list_t *snrlp = NULL;
scds_port_list_t *portlist = NULL;
struct stat stat_buf;
char *start_cmd_prog[] = { "" };
/* int i; */
int do_cmd_checks = 1;
char *start_cmd="/sbin/cupsd";
char cmd[CMD_SIZE];

	/* Process arguments passed by RGM and initialize syslog */
	if (scds_initialize(&scds_handle, argc, argv) != SCHA_ERR_NOERR) {
		scds_syslog(LOG_ERR, "Data Service handle failed to initialize!");
		return (1);
	}

	/* Are we running with a libdsdev that does not have scds_hasp_check? */
	hasp_check_p = (scha_err_t)dlsym(RTLD_DEFAULT, "scds_hasp_check");
	if (hasp_check_p == NULL) {
		/* fake a call to scds_hasp_check() */
		err_code = SCHA_ERR_NOERR;
		hasp_status = SCDS_HASP_NO_RESOURCE;
	} else {
		/* actually check for HAStoragePlus resources */
		err_code = scds_hasp_check(scds_handle, &hasp_status);
	}

	if (err_code != SCHA_ERR_NOERR) {
		/* scha_hasp_check() logs a message to syslog when it fails */
		scds_syslog(LOG_ERR,"HAStoragePlus dependency problem! Error: %s",scds_error_string(err_code));
		scds_close(&scds_handle);
		return(err_code);
	}

	switch (hasp_status) {

	case SCDS_HASP_NO_RESOURCE:
		/*
		 * We do not depend on any HAStoragePlus resources
		 */
		scds_syslog(LOG_NOTICE,
		    "This resource does not depend on any HAStoragePlus "
		    "resources. Proceeding with normal checks.");
		do_cmd_checks = 1;
		break;

	case SCDS_HASP_ERR_CONFIG:
		/*
		 * Configuration error, HAStoragePlus resource is
		 * in a different RG. Fail the validation.
		 */
		scds_syslog(LOG_ERR,
		    "One or more of the HAStoragePlus resources that "
		    "this resource depends on is in a different resource "
		    "group. Failing validate method configuration checks.");
		err_code = 1;
		scds_close(&scds_handle);
		return(err_code);

	case SCDS_HASP_NOT_ONLINE:
		/*
		 * There is at least one HAStoragePlus resource not
		 * online anywhere.
		 */
		scds_syslog(LOG_ERR,
		    "One or more of the HAStoragePlus resources that "
		    "this resource depends on is not online anywhere. "
		    "Failing validate method.");
		err_code = 1;
                scds_close(&scds_handle);
                return(err_code);

	case SCDS_HASP_ONLINE_NOT_LOCAL:
		/*
		 * Not all HAStoragePlus we need, are online locally.
		 */
		scds_syslog(LOG_NOTICE,
		    "All the HAStoragePlus resources that this resource "
		    "depends on are not online on the local node. "
		    "Skipping the checks for the existence and permissions "
		    "of the start/stop/probe commands.");
		do_cmd_checks = 0;
		break;

	case SCDS_HASP_ONLINE_LOCAL:
		/*
		 * All HAStoragePlus resources we need are available on
		 * this node.
		 */
		scds_syslog(LOG_NOTICE,
		    "All the HAStoragePlus resources that this resource "
		    "depends on are online on the local node. "
		    "Proceeding with the checks for the existence and "
		    "permissions of the start/stop/probe commands.");
		do_cmd_checks = 1;
		break;

	default:
		/* Unknown status code */
		scds_syslog(LOG_ERR, "Unknown status code %d.", hasp_status);
		err_code = 1;
		scds_close(&scds_handle);
		return(err_code);
	}


	/*
	 * If do_cmd_checks is set to 1, it means that the HAStorage
	 * resource is online on the local node. Therefore, proceed with
	 * the checks for the existence and permissions of the
	 * start commands.
	 */

	if (do_cmd_checks == 1) {
		err_code = scds_get_ext_property(scds_handle, "CUPS_dir", SCHA_PTYPE_STRINGARRAY, &cups_ext_prop);
		if( err_code != SCHA_ERR_NOERR ){
        		scds_syslog(LOG_ERR,"Failed to get Config directory! Error: %s",scds_error_string(err_code));
        		scds_close(&scds_handle);
        		return(err_code);
		}

		strcpy(start_cmd_prog[0],cups_ext_prop->val.val_strarray->str_array[0]);
		strcat(start_cmd_prog[0], start_cmd);
		strcpy(cmd, start_cmd_prog[0]);


		if(stat(cmd, &stat_buf) != 0) {
			scds_syslog(LOG_ERR,
			    "Cannot access the %s command <%s> : <%s>",
			    "start", cmd, strerror(errno));
			err_code = 1;
		}

		if(!(stat_buf.st_mode & S_IXUSR)) {
			scds_syslog(LOG_ERR,
			    "The %s command does not have execute "
			    "permissions: <%s>", "start", cmd);
			err_code = 1;
		}
		
		scds_free_ext_property(cups_ext_prop);
	}

	/*
	 * Network aware service should have at least one port specified
	 */

	err_code = scds_get_port_list(scds_handle, &portlist);
	if(err_code != SCHA_ERR_NOERR) {
		scds_syslog(LOG_ERR,
		    "Failed to retrieve the resource property %s: %s.",
		    SCHA_PORT_LIST, scds_error_string(err_code));
	}

	if(portlist == NULL || portlist->num_ports < 1) {
		scds_syslog(LOG_ERR, "Property %s is not set.", SCHA_PORT_LIST);
		err_code = 1;
	}

	/*
	 * Return an error if there is an error when trying to get the
	 * available network address resources for this resource
	 */
	err_code = scds_get_rs_hostnames(scds_handle, &snrlp);
	if (err_code != SCHA_ERR_NOERR) {
		scds_syslog(LOG_ERR,
		    "Error in trying to access the configured network "
		    "resources : %s.", scds_error_string(err_code));
		err_code = 1;
	}

	/* Return an error if there are no network address resources */
	if (snrlp == NULL || snrlp->num_netresources == 0) {
		scds_syslog(LOG_ERR, "No network address resource in resource group.");
		err_code = 1;
	}

	scds_free_net_list(snrlp);
	scds_free_port_list(portlist);

	/* Free up all the memory allocated by scds_initialize */
	(void) scds_close(&scds_handle);

	/* Return the result of validate method */
	return(err_code);

}
